///***************************************************************************************
// RFM69 Transceiver
// Do SPI bit banger
// Jed Margolin  2/1/2021.
// Built with Code Composer Studio v10
//***************************************************************************************
#include <msp430.h>
#include <rfm69.h>
//================================================
unsigned char spi_read(unsigned char address)
{
    int n;
    unsigned char data, temp;

    P1OUT &= ~BIT5;          // make sure SCLK is low
    wait_rfm1();
    P1OUT &= ~BIT4;         // CS low

// read the address
    address &= 0x7F;       // Read = 0

    for(n=0; n<8; n++)
    {
        if(address & 0x80) {P1OUT |= BIT6;}        // SDO high
        else {P1OUT &= ~BIT6;}                     // SDO low
        wait1();

        P1OUT |= BIT5;      // make SCLK high

        wait_rfm1();

        address = address<<1;

        P1OUT &= ~BIT5;      // make SCLK low
    }

// now get the data
    data = 0;
    wait_rfm1();

    for(n=0; n<8; n++)
    {

       P1OUT |= BIT5;      // make SCLK high

       wait_rfm1();
       data = data <<1;

       temp = P1IN & BIT7;
       if(temp) {data = data | 0x01;}

       P1OUT &= ~BIT5;        // SCLK low

       wait_rfm1();
    }

    P1OUT |= BIT4;  //  /CS high

    return(data);
}
//====================================================
void spi_write(unsigned char address, unsigned char data)
{
    int n;

    P1OUT &= ~BIT6;          // make sure SCLK is low

    address |= 0x80;         // Write = 1

    wait_rfm1();

    P1OUT &= ~BIT4;         // CS low

    for(n=0; n<8; n++)
    {
        if(address & 0x80) {P1OUT |= BIT6;}        // SDO high
        else {P1OUT &= (~BIT6);}                   // SDO low
        wait_rfm1();

        P1OUT |= BIT5;      // make SCLK high

        wait_rfm1();

        address = address<<1;

        P1OUT &= ~BIT5;      // make SCLK low
    }

    for(n=0; n<8; n++)
    {
        if(data & 0x80) {P1OUT |= BIT6;}        // SDO high
        else {P1OUT &= (~BIT6);}                // SDO low
        wait_rfm1();

        P1OUT |= BIT5;      // make SCLK high

        wait_rfm1();

        data = data<<1;

        P1OUT &= ~BIT5;      // make SCLK low
    }

    P1OUT |= BIT4;         // CS high
}
//=====================================================
int wait_rfm_ready()
{
    int n, timeout;
    unsigned char busy, data;

    busy = 1;
    n = 1000;
    timeout = 0;

    while(busy)
    {
       data = spi_read(0xF3);
       if( !(data & 0x08)) {busy = 0;}
       if(n == 0)  {timeout = 1;}
       n--;
    }
    return timeout;
}
//=======================================================
void wait_rfm1()
{
    asm(" nop");    // for debugger
    return;
}
//=========================
void rfm69_test1(void)
{
    unsigned char dataread[8];

    spi_write(RegFrfMsb,0x6C);      // MSB of the RF carrier frequency   433.92 MHz 0x6C
    spi_write(RegFrfMid,0x7A);      // Middle Byte of the RF carrier frequency      0x7A
    spi_write(RegFrfLsb,0xE1);      // LSB of the RF carrier frequency              0xE1

    dataread[0] = spi_read(RegFrfMsb);      // MSB of the RF carrier frequency   433.92 MHz 0x6C
    dataread[1] = spi_read(RegFrfMid);      // Middle Byte of the RF carrier frequency      0x7A
    dataread[2] = spi_read(RegFrfLsb);      // LSB of the RF carrier frequency              0xE1

    asm(" nop");    // for debugger
}
//=========================
void rfm69_test2(void)
{
    unsigned char dataread[8];

    dataread[0] = spi_read(RegFrfMsb);      // MSB of the RF carrier frequency   433.92 MHz 0x6C  Default 0xE4
    dataread[1] = spi_read(RegFrfMid);      // Middle Byte of the RF carrier frequency      0x7A  Default 0xC0
    dataread[2] = spi_read(RegFrfLsb);      // LSB of the RF carrier frequency              0xE1  Default 0x00

    asm(" nop");    // for debugger
}
//============================
void rfm69_test3(void)
{
    unsigned char data, n, dataread[8];

    data = 0x00;
    for(n = 8; n>0; n--)
    {
       spi_write(RegFifo, data++);
    }

    wait1ms();
    for(n = 0; n<64; n++)
    {
       dataread[n] = spi_read(RegFifo);
    }
    asm(" nop");    // for debugger
}
//=========================
// OOK
void rfm69_init(void)
{

   P1OUT &= ~BIT2;

   spi_write(RegOpMode,0x00);      // Sleep

// Transmitter
   spi_write(RegFrfMsb,0x6C);   // MSB of the RF carrier frequency   433.92 MHz 0x6C   434.2 MHz 0x6C    433.8 MHz  0x6C
   spi_write(RegFrfMid,0x73);   // Middle Byte of the RF carrier frequency      0x7A             0x8C               0x73
   spi_write(RegFrfLsb,0x33);   // LSB of the RF carrier frequency              0xE1             0xCD               0x33

   spi_write(RegDataModul,0x68);   // OOK, continuous without Synchronizer

   spi_write(RegDioMapping1,0x00); // Dio2 for transmit

   spi_write(RegPaLevel,0xFF); // default 0x9F: Enables PA0, connected to RFIO and LNA; Output power setting,
//                                     with 1 dB steps Pout = -18 + OutputPower [dBm] , with PA0

    spi_write(RegOpMode,0x0C);      // Transmitter mode (TX)
}
//==============================================
void rfm69_test4()
{

   P1OUT &= ~BIT2;

   spi_write(RegOpMode,0x00);      // Sleep

// Transmitter Test   434.2 MHz
   spi_write(RegFrfMsb,0x6C);      // MSB of the RF carrier frequency   433.92 MHz 0x6C
   spi_write(RegFrfMid,0x8C);      // Middle Byte of the RF carrier frequency      0x7A
   spi_write(RegFrfLsb,0xCD);      // LSB of the RF carrier frequency              0xE1

//        spi_write(RegFdevMsb,0x00);     // MSB of the frequency deviation: [5 KHz]
//        spi_write(RegFdevLsb,0x52);     // LSB of the frequency deviation

//        spi_write(RegPreambleMsb,0);    // Size of the preamble to be sent (from TxStartCondition fulfilled). (MSB byte)
//        spi_write(RegPreambleLsb,3);    // Size of the preamble to be sent (from TxStartCondition fulfilled). (LSB byte)

//        spi_write(RegSyncConfig,0x99);  // Sync word generation; Sync word size = 2
//        spi_write(RegSyncValue1,0x01);  // 1st byte of Sync word
//        spi_write(RegSyncValue2,0x02);  // 2nd byte of Sync word

   spi_write(RegDataModul,0x68);   // OOK, continuous without Synchronizer
//        spi_write(RegDataModul,0x08);   // Packet Mode, OOK, No Modulation Shaping

   spi_write(RegDioMapping1,0x00); // Dio2 for transmit 0x0C?

   spi_write(RegPaLevel,0x9F); // Enables PA0, connected to RFIO and LNA; Output power setting,
//                                     with 1 dB steps Pout = -18 + OutputPower [dBm] , with PA0

//      spi_write(RegDioMapping1,0xx);  // work on this

//        flags = spi_read(RegIrqFlags1);

//        spi_write(RegPacketConfig1,0x00);  // Vanilla
//        spi_write(RegPacketConfig1,0x50);  // Fixed Length; Whitening; Enable CRC

//        spi_write(RegPayloadLength,16);    // payload length

//        spi_write(NodeAddress,0x55);     // Node address used in address filtering

//        spi_write(RegBroadcastAdrs,0x33);  // Broadcast address used in address filtering

//        spi_write(RegFifoThresh,0x00);  // Defines the condition to start packet transmission; the number of bytes in the FIFO exceeds FifoThreshold
//      spi_write(RegFifoThresh,0x80);  // Defines the condition to start packet transmission; at least one byte in the FIFO

   spi_write(RegOpMode,0x0C);      // Transmitter mode (TX)
//   spi_write(RegDioMapping1,0x0C); // Dio2 for transmit

   while(1)
   {
      wait1ms();           // 500 Hz
      P1OUT ^= BIT2;
   }

//    address = 0x10;
//    spi_writeword(address,0x78);

//    address = 0x07;     // Frequency MSB
//    freq_msb = spi_read(address);

//    address = 0x10;
//    for(k=0; k<3; k++)
//    {
//       wait_rfm_ready();
//       rfmdata[k]= spi_read(address++);
//    }
//end transmitter test

}
//==============================================
